from datetime import datetime, timezone

import Apifon.sdk.Mookee as mookee
import json
from Apifon.sdk.Model.IMAction import IMAction
from Apifon.sdk.Model.IMChannel import IMChannel
from Apifon.sdk.Model.ImRequest import ImRequest
from Apifon.sdk.Model.LandingProperties import LandingProperties
from Apifon.sdk.Model.MessageContent import MessageContent
from Apifon.sdk.Model.RemoveRequest import RemoveRequest
from Apifon.sdk.Model.SmsRequest import SmsRequest
from Apifon.sdk.Model.SubscriberInformation import SubscriberInformation
from Apifon.sdk.Model.OtpRequest import OtpRequest
from Apifon.sdk.Resource.BalanceResource import BalanceResource
from Apifon.sdk.Resource.IMResource import IMResource
from Apifon.sdk.Resource.SMSResource import SMSResource
from Apifon.sdk.Resource.OTPResource import OTPResource


def remove_nones(var):
    for i in list(var):
        if not var[i] and var[i] is not 0:
            var.pop(i)
        elif type(var[i]) is dict:
            remove_nones(var[i])
        elif type(var[i]) is list:
            var[i] = [x for x in var[i] if x != 20]
            for j in var[i]:
                if type(j) is dict:
                    remove_nones(j)


def print_json(output):
    json_string = json.dumps(output, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    json_dict = json.loads(json_string)
    remove_nones(json_dict)
    json_string = json.dumps(json_dict, default=lambda o: o.__dict__, sort_keys=True, indent=4)
    print(json_string)

# This is a complete example on how to use the APIFON SDK for Python.


Mookee = mookee.Mookee.get_instance()

'''
First things first, we need to set our credentials that we got from
the Mookee platform.
'''
Mookee.add_credentials("sms", "your_client_token", "your_secret_key")
Mookee.add_credentials("viber", "your_client_token", "your_secret_key")
Mookee.add_credentials("otp", "Your_OTP_Token", "Your_OTP_Secret_Key")
'''
These are our Resources.
'''
smsResource = SMSResource()
imResource = IMResource()
otpResource = OTPResource()

'''
These are our Requests.
'''
smsRequest = SmsRequest()
imRequest = ImRequest()
otpRequest = OtpRequest()
removeRequest = RemoveRequest()

'''
Here we generate the date that scheduled messages will be sent.
'''
dateTime = list(datetime.now(timezone.utc).strftime('%Y-%m-%dT%H:%M:%S'))
dateTime[2] = "3"
dateTime = "".join(dateTime)

'''
Here we create our subscribers.
'''
names = ["John", "George", "Alex", "Helen"]
names2 = ["Kate", "Julie", "Paul", "Mary"]
nums = ["306999999999"]

'''
Here we define the content of our Request.
'''
subscribers = list()
imChannel = IMChannel()
imAction = IMAction()
messageContent = MessageContent()
callbackUrl = "https://yourserver/callback"
replyUrl = "https://yourserver/replyurl"

choice = "1"


while choice is not 0:
    print("------ Menu ------")
    print("0.  Exit")
    print("1.  Send Simple SMS")
    print("2.  Send Complete SMS")
    print("3.  Send Scheduled SMS")
    print("4.  Remove Scheduled SMS")
    print("5.  Send SMS With Landing Page")
    print("6.  Case 1: Send IM With Text")
    print("7.  Case 2: Send IM With Image")
    print("8.  Case 3: Send IM With Text and Button")
    print("9.  Case 4: Send IM With Text, Image and Button")
    print("10. Send Complete IM without fallback")
    print("11. Send Scheduled IM")
    print("12. Remove Scheduled IM")
    print("13. Send IM With Landing Page")
    print("14. Send IM With Fallback SMS")
    print("15. Send IM With Fallback Landing Page SMS")
    print("16. Send 2-Way IM")
    print("17. Send Balance Request")
    print("18. Send Simple OTP Create Request")
    print("19. Send Complete OTP Create Request")
    print("20. Send OTP Verification Request")
    choice = input("Choice: ")
    try:
        choice = int(choice)
    except ValueError:
        print("Please enter an int.")

    '''
    Every request has the same structure: 
     *First we set the Credential we want to use. 
     *Then we build the components of the Request. 
     *Finally we build the Request and send it. 
     *We print the response we get in console.
    '''
    # Exit Loop
    if choice is 0:
        print("Exiting...")

    # This is a simple SMS Request.
    elif choice is 1:
        Mookee.set_active_credential("sms")

        smsRequest = SmsRequest()

        msgContent = MessageContent()
        msgContent.text = "Hello There!"
        msgContent.sender_id = "Apifon"

        smsRequest.add_str_subscribers(nums)
        smsRequest.message = msgContent
        smsRequest.callback_url = callbackUrl

        print("Sending Request to Endpoint: /services/sms/send")
        print("------ SMS Request Body ------")
        print_json(smsRequest)
        response = smsResource.send(smsRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # In this SMS example we are taking advantage of every property that
    # can be changed in a Request.
    elif choice is 2:
        Mookee.set_active_credential("sms")

        smsRequest = SmsRequest()

        msgContent = MessageContent()
        msgContent.text = "Hello {name}!"
        msgContent.dc = 0
        msgContent.sender_id = "Apifon"

        subscribers = list()
        for num in nums:
            sub = SubscriberInformation()
            sub.number = num
            sub.custom_id = "222"
            sub.add_params("name", names[len(subscribers)])
            subscribers.append(sub)

        smsRequest.add_subscribers(subscribers)
        smsRequest.reference_id = "reference"
        smsRequest.message = msgContent
        smsRequest.callback_url = callbackUrl
        smsRequest.tte = "86400"

        print("Sending Request to Endpoint: /services/sms/send")
        print("------ SMS Request Body ------")
        print_json(smsRequest)
        response = smsResource.send(smsRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # Here we send a scheduled SMS Request.
    elif choice is 3:
        Mookee.set_active_credential("sms")

        smsRequest = SmsRequest()

        msgContent = MessageContent()
        msgContent.text = "Hello There!"
        msgContent.sender_id = "Apifon"

        smsRequest.add_str_subscribers(nums)
        smsRequest.message = msgContent
        smsRequest.callback_url = callbackUrl
        smsRequest.date = dateTime

        print("Sending Request to Endpoint: /services/sms/send")
        print("------ SMS Request Body ------")
        print_json(smsRequest)
        response = smsResource.send(smsRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # This is an SMS Deletion case.
    elif choice is 4:
        Mookee.set_active_credential("sms")

        deletionType = ""
        while deletionType != "0" and deletionType != "1" and deletionType != "2":
            deletionType = input("Do you want to delete scheduled messages using Request ID or Reference ID?\nEnter [0] for Request ID, [1] for Reference ID or [2] to exit: ")
            if deletionType != "0" and deletionType != "1" and deletionType != "2":
                print("Invalid Option.")
        if deletionType != "2":
            removeRequest = RemoveRequest()
            if deletionType == "0":
                removeRequest.field = "REQUEST_ID"
                removeRequest.id = input("Please enter the Request ID of the scheduled message you want to delete: ")
            else:
                removeRequest.field = "REFERENCE_ID"
                removeRequest.id = input("Please enter the Reference ID of the scheduled message you want to delete: ")
            print("Sending Request to Endpoint: /services/sms/remove")
            print("------ SMS Remove Request Body ------")
            print_json(removeRequest)
            delResp = smsResource.remove(removeRequest)
            print("------ SMS Remove Request Response ------\n" + str(delResp))

    # In this example we are sending an SMS Request with a landing page.
    # {apifon_lp} will later be replaced with the URL property of the
    # landing page.
    elif choice is 5:
        Mookee.set_active_credential("sms")

        smsRequest = SmsRequest()

        msgContent = MessageContent()
        msgContent.text = "Hello {name}! {apifon_lp}"
        msgContent.dc = 0
        msgContent.sender_id = "Apifon"

        subscribers = list()
        for num in nums:
            sub = SubscriberInformation()
            sub.number = num
            sub.custom_id = "222"
            sub.add_params("name", names[len(subscribers)])

            lp = LandingProperties()
            lp.url = "https://mookee.apifon.com/user/login/"
            lp.add_data("name", names2[len(subscribers)])
            lp.redirect = True
            sub.set_landing_page(lp)

            subscribers.append(sub)

        smsRequest.add_subscribers(subscribers)
        smsRequest.reference_id = "reference"
        smsRequest.message = msgContent
        smsRequest.callback_url = callbackUrl
        smsRequest.tte = "86400"

        print("Sending Request to Endpoint: /services/sms/send")
        print("------ SMS Request Body ------")
        print_json(smsRequest)
        response = smsResource.send(smsRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # Viber Case 1: Send IM that contains only a text.
    elif choice is 6:
        Mookee.set_active_credential("viber")

        imRequest = ImRequest()

        imChannel = IMChannel()
        imChannel.text = "50% discount on our membership fee if you renew your subscription today!"
        imChannel.sender_id = "Mookee"

        imRequest.add_str_subscribers(nums)
        imRequest.callback_url = callbackUrl
        imRequest.add_im_channel(imChannel)

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # Viber Case 2: Send IM that contains only an image.
    elif choice is 7:
        Mookee.set_active_credential("viber")

        imRequest = ImRequest()

        imChannel = IMChannel()
        imChannel.sender_id = "Mookee"
        imChannel.add_image("http://assets.apifon.com/images/boy_girl.jpg")

        imRequest.add_str_subscribers(nums)
        imRequest.callback_url = callbackUrl
        imRequest.add_im_channel(imChannel)

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # Viber Case 3: Send IM that contains text and action button.
    elif choice is 8:
        Mookee.set_active_credential("viber")

        imChannel = IMChannel()
        imRequest = ImRequest()

        imAction = IMAction()
        imAction.title = "Renew now!"
        imAction.target_url = "https://www.apifon.com"

        imChannel.text = "50% discount on our membership fee if you renew your subscription today!"
        imChannel.sender_id = "Mookee"
        imChannel.add_action(imAction)

        imRequest.add_str_subscribers(nums)
        imRequest.callback_url = callbackUrl
        imRequest.add_im_channel(imChannel)

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # Viber Case 4: Send IM that contains text, image and action button.
    elif choice is 9:
        Mookee.set_active_credential("viber")

        imChannel = IMChannel()
        imRequest = ImRequest()

        imAction = IMAction()
        imAction.title = "Renew now!"
        imAction.target_url = "https://www.apifon.com"

        imChannel.text = "50% discount on our membership fee if you renew your subscription today!"
        imChannel.sender_id = "Mookee"
        imChannel.add_image("http://assets.apifon.com/images/boy_girl.jpg")
        imChannel.add_action(imAction)

        imRequest.add_str_subscribers(nums)
        imRequest.callback_url = callbackUrl
        imRequest.add_im_channel(imChannel)

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # Complete Viber example.
    elif choice is 10:
        Mookee.set_active_credential("viber")

        imChannel = IMChannel()
        imRequest = ImRequest()

        imAction = IMAction()
        imAction.title = "Renew now!"
        imAction.target_url = "https://www.apifon.com"

        imChannel.id = "channelID"
        imChannel.text = "{name}! 50% discount on our membership fee if you renew your subscription today!"
        imChannel.sender_id = "Mookee"
        imChannel.add_image("http://assets.apifon.com/images/boy_girl.jpg")
        imChannel.add_action(imAction)
        imChannel.ttl = 86400
        imChannel.expiry_text = "Message Expired!!"

        subscribers = list()
        for num in nums:
            sub = SubscriberInformation()
            sub.number = num
            sub.custom_id = "222"
            sub.add_params("name", names[len(subscribers)])

            subscribers.append(sub)

        imRequest.add_subscribers(subscribers)
        imRequest.reference_id = "reference"
        imRequest.callback_url = callbackUrl
        imRequest.add_im_channel(imChannel)

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # Here we send a scheduled IM Request.
    elif choice is 11:
        Mookee.set_active_credential("viber")

        imChannel = IMChannel()
        imRequest = ImRequest()

        imAction = IMAction()
        imAction.title = "Renew now!"
        imAction.target_url = "https://www.apifon.com"

        imChannel.text = "50% discount on our membership fee if you renew your subscription today!"
        imChannel.sender_id = "Mookee"
        imChannel.add_image("http://assets.apifon.com/images/boy_girl.jpg")
        imChannel.add_action(imAction)

        imRequest.add_str_subscribers(nums)
        imRequest.callback_url = callbackUrl
        imRequest.add_im_channel(imChannel)
        imRequest.date = dateTime

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # This is an IM Deletion case.
    elif choice is 12:
        Mookee.set_active_credential("viber")

        deletionType = ""
        while deletionType != "0" and deletionType != "1" and deletionType != "2":
            deletionType = input("Do you want to delete scheduled messages using Request ID or Reference ID?\nEnter [0] for Request ID, [1] for Reference ID or [2] to exit: ")
            if deletionType != "0" and deletionType != "1" and deletionType != "2":
                print("Invalid Option.")
        if deletionType != "2":
            removeRequest = RemoveRequest()
            if deletionType == "0":
                removeRequest.field = "REQUEST_ID"
                removeRequest.id = input(
                    "Please enter the Request ID of the scheduled message you want to delete: ")
            else:
                removeRequest.field = "REFERENCE_ID"
                removeRequest.id = input(
                    "Please enter the Reference ID of the scheduled message you want to delete: ")
            print("Sending Request to Endpoint: /services/im/remove")
            print("------ IM Remove Request Body ------")
            print_json(removeRequest)
            delResp = imResource.remove(removeRequest)
            print("------ IM Remove Request Response ------\n" + str(delResp))

    # In this example we send a Viber message containing a landing page.
    elif choice is 13:
        Mookee.set_active_credential("viber")

        imChannel = IMChannel()
        imRequest = ImRequest()

        imAction = IMAction()
        imAction.title = "Renew now!"
        imAction.target_url = "https://www.apifon.com"

        imChannel.text = "{name}! 50% discount on our membership fee if you renew your subscription today!"
        imChannel.sender_id = "Mookee"
        imChannel.add_image("http://assets.apifon.com/images/boy_girl.jpg")
        imChannel.add_action(imAction)

        subscribers = list()
        for num in nums:
            sub = SubscriberInformation()
            sub.number = num
            sub.custom_id = "222"
            sub.add_params("name", names[len(subscribers)])

            lp = LandingProperties()
            lp.url = "https://mookee.apifon.com/user/login/"
            lp.add_data("name", names2[len(subscribers)])
            lp.redirect = True
            sub.set_landing_page(lp)

            subscribers.append(sub)

        imRequest.add_subscribers(subscribers)
        imRequest.callback_url = callbackUrl
        imRequest.add_im_channel(imChannel)

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # In this example we send a Viber message with a fallback SMS, in
    # case Viber delivery fails.
    elif choice is 14:
        Mookee.set_active_credential("viber")

        imChannel = IMChannel()
        imRequest = ImRequest()

        imAction = IMAction()
        imAction.title = "Renew now!"
        imAction.target_url = "https://www.apifon.com"

        imChannel.text = "50% discount on our membership fee if you renew your subscription today!"
        imChannel.sender_id = "Mookee"
        imChannel.add_image("http://assets.apifon.com/images/boy_girl.jpg")
        imChannel.add_action(imAction)

        msgContent = MessageContent()
        msgContent.text = "Hello There!"
        msgContent.sender_id = "Apifon"

        imRequest.add_str_subscribers(nums)
        imRequest.message = msgContent
        imRequest.callback_url = callbackUrl
        imRequest.add_im_channel(imChannel)

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # In this example we send a Viber message with a landing page
    # fallback SMS, in case Viber delivery fails.
    elif choice is 15:
        Mookee.set_active_credential("viber")

        imChannel = IMChannel()
        imRequest = ImRequest()

        imAction = IMAction()
        imAction.title = "Renew now!"
        imAction.target_url = "https://www.apifon.com"

        imChannel.text = "50% discount on our membership fee if you renew your subscription today!"
        imChannel.sender_id = "Mookee"
        imChannel.add_image("http://assets.apifon.com/images/boy_girl.jpg")
        imChannel.add_action(imAction)

        subscribers = list()
        for num in nums:
            sub = SubscriberInformation()
            sub.number = num
            sub.custom_id = "222"
            sub.add_params("name", names[len(subscribers)])

            lp = LandingProperties()
            lp.url = "79424c5574569d26098457p6"
            lp.add_data("name", names2[len(subscribers)])
            lp.method = "POST"
            lp.redirect = False
            sub.set_landing_page(lp)

            subscribers.append(sub)

        msgContent = MessageContent()
        msgContent.text = "Hello {name}! {apifon_lp}"
        msgContent.sender_id = "Apifon"

        imRequest.add_subscribers(subscribers)
        imRequest.message = msgContent
        imRequest.callback_url = callbackUrl
        imRequest.add_im_channel(imChannel)

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # This is a 2-way Viber example. Any replies sent by the
    # message recipient will generate a reply-message callback
    # on the replyURL. Note that this request would require use of
    # a Viber Sender ID with 2-way messaging enabled. 2-way
    # messaging can be enabled on a Sender ID via our platform.
    elif choice is 16:
        Mookee.set_active_credential("viber")

        imChannel = IMChannel()
        imRequest = ImRequest()

        imAction = IMAction()
        imAction.title = "Renew now!"
        imAction.target_url = "https://www.apifon.com"

        imChannel.text = "50% discount on our membership fee if you renew your subscription today! Would you like to renew your subscription? Would you like to renew your subscription?"
        imChannel.sender_id = "Mookee"
        imChannel.add_image("http://assets.apifon.com/images/boy_girl.jpg")
        imChannel.add_action(imAction)

        imRequest.add_str_subscribers(nums)
        imRequest.callback_url = callbackUrl
        imRequest.reply_url = replyUrl
        imRequest.add_im_channel(imChannel)

        print("Sending Request to Endpoint: /services/im/send")
        print("------ IM Request Body ------")
        print_json(imRequest)
        response = imResource.send(imRequest)
        print("------ Gateway Response Body ------")
        print_json(response)

    # This is a Balance Request example in which we are informed about our
    # remaining Balance and our Plafon.
    elif choice is 17:
        Mookee.set_active_credential("sms")

        balanceResource = BalanceResource()
        print("Sending Request to Endpoint: /services/balance")
        print("------ Balance Request Body ------\nEmpty Body")
        balanceResp = balanceResource.send()
        print("------ Balance Response Body ------")
        print_json(balanceResp)

    # This is a basic OTP Create Request example. We are only setting a phone
    # number and a reference_id. Reference_id is important because it links to
    # the generated OTP.
    elif choice is 18:
        Mookee.set_active_credential("otp")

        otpRequest = OtpRequest()

        otpRequest.reference_id = input("Enter a reference_id for your OTP create request: ")
        otpRequest.subscriber = nums[0]

        print("Sending Request to Endpoint: /services/otp/create")
        print("------ OTP Create Request Body ------")
        print_json(otpRequest)
        otpResp = otpResource.create(otpRequest)
        print("------ OTP Create Response Body ------")
        print_json(otpResp)

    # This is a complete OTP Create Request example. We are specifying every
    # field of the request to our preferences.
    elif choice is 19:
        Mookee.set_active_credential("otp")

        otpRequest = OtpRequest()

        msgContent = MessageContent()
        msgContent.text = "Hello There! Here is your OTP: {apifon_otp}"
        msgContent.sender_id = "Apifon"

        otpRequest.reference_id = input("Enter a reference_id for your OTP create request: ")
        otpRequest.subscriber = nums[0]
        otpRequest.code_length = 6
        otpRequest.code_type = "numeric"
        otpRequest.callback_url = callbackUrl
        otpRequest.expire = 300
        otpRequest.message = msgContent

        print("Sending Request to Endpoint: /services/otp/create")
        print("------ OTP Create Request Body ------")
        print_json(otpRequest)
        otpResp = otpResource.create(otpRequest)
        print("------ OTP Create Response Body ------")
        print_json(otpResp)

    # This is an OTP Verification Request example. When a subscriber is using
    # the OTP that we generated for him, we want to verify that the OTP is valid.
    # We are using the reference_id that we used to generate the OTP and the OTP that
    # we are provided to make this verification.
    elif choice is 20:
        Mookee.set_active_credential("otp")
        otpReference = input("Enter the reference_id of the OTP you want to verify: ")
        otpCode = input("Enter the OTP you want to verify: ")

        print("Sending Request to Endpoint: /services/otp/verify/" + otpReference + "/" + otpCode)
        print("------ OTP Verify Request Body ------\nEmpty Body")
        otpVerResp = otpResource.verify(otpReference, otpCode)
        print("------ OTP Verify Response Body ------")
        print_json(otpVerResp)
