import json

from Apifon.sdk.Resource.Helpers.AbstractHttpEtag import AbstractHttpEtag
from Apifon.sdk.Resource.Helpers.AbstractJsonSerializable import AbstractJsonSerializable
from Apifon.sdk.Resource.Helpers.GroupOptionsOrientation import GroupOptionsOrientation
from Apifon.sdk.Resource.Helpers.GroupType import GroupType
from Apifon.sdk.Resource.Helpers.SelectionType import SelectionType
from Apifon.sdk.Response.GroupResponse import GroupResponse


class GroupRequestBuilder(AbstractHttpEtag):

    def __init__(self, name, type, list_id, group_field):
        self.list_id = list_id
        self.group_field = group_field
        self.name = name
        self.type = type
        self.internal = None
        self.selectionType = None
        self.groupOptionsOrientation = None

    @staticmethod
    def get_add_request_builder(name, type, list_id):
        if type is None or not GroupType.has_value(type):
            raise TypeError("Invalid group type. Should be of type GroupType.value")
        if name is None or not isinstance(name, str):
            raise TypeError("Invalid name. Should be of type str")
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        return GroupRequestBuilder(name, type, list_id, None)

    @staticmethod
    def get_update_request_builder(name, type, list_id, group_field):
        if type is None or not GroupType.has_value(type):
            raise TypeError("Invalid group type. Should be of type GroupType.value")
        if name is None or not isinstance(name, str):
            raise TypeError("Invalid name. Should be of type str")
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if group_field is None or not isinstance(group_field, str):
            raise TypeError("Invalid group field. Should be of type str")
        return GroupRequestBuilder(name, type, list_id, group_field)

    @staticmethod
    def get_update_request_builder_from_response(list_id, groupResponse):
        if groupResponse is None or not isinstance(groupResponse, GroupResponse):
            raise TypeError("Invalid group response type. Should be of type GroupResponse")
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")

        groupToUpdate = GroupRequestBuilder(groupResponse.name, groupResponse.type, list_id, groupResponse.field)
        groupToUpdate.internal = groupResponse.internal
        groupToUpdate.groupOptionsOrientation = groupResponse.groupOptionsOrientation
        groupToUpdate.selectionType = groupResponse.selectionType

        return groupToUpdate

    def with_name(self, name):
        if name is None or not isinstance(name, str):
            raise TypeError("Invalid name. Should be of type str")
        self.name = name
        return self

    def with_type(self, type):
        if type is None or not GroupType.has_value(type):
            raise TypeError("Invalid group type. Should be of type GroupType.value")
        self.type = type
        return self

    def with_internal(self, internal):
        if internal is None or not isinstance(internal, bool):
            raise TypeError("Invalid internal. Should be of type bool")
        self.internal = internal
        return self

    def with_selectionType(self, selectionType):
        if selectionType is None or not SelectionType.has_value(selectionType):
            raise TypeError("Invalid selection type. Should be of type SelectionType.value")
        self.selectionType = selectionType
        return self

    def with_groupOptionsOrientation(self, groupOptionsOrientation):
        if groupOptionsOrientation is None or not GroupOptionsOrientation.has_value(groupOptionsOrientation):
            raise TypeError("Invalid groupOptionsOrientation. Should be of type GroupOptionsOrientation.value")
        self.groupOptionsOrientation = groupOptionsOrientation
        return self

    def build(self):
        return self.GroupRequest(self.name, self.type, self.list_id, self.group_field, self.internal, self.selectionType
                                 , self.groupOptionsOrientation)

    def get_etag(self):
        pass

    class GroupRequest(AbstractJsonSerializable):

        def __init__(self, name, type, list_id, group_field, internal, selectionType, groupOptionsOrientation):
            self.list_id = list_id
            self.group_field = group_field
            self.name = name
            self.type = type
            self.internal = internal
            self.selectionType = selectionType
            self.groupOptionsOrientation = groupOptionsOrientation

        # override
        def get_body(self):

            json_string = json.dumps(self, default=lambda o: o.__dict__)
            json_dict = json.loads(json_string)
            self.remove_nones(json_dict)
            json_string = json.dumps(json_dict)
            return json_string

        def remove_nones(self, var):
            for i in list(var):
                if var[i] is None or (isinstance(var[i], dict) and not bool(var[i])):
                    var.pop(i)
                elif var[i] == self.list_id:
                    var.pop(i)
                elif var[i] == self.group_field:
                    var.pop(i)
                elif type(var[i]) is dict:
                    self.remove_nones(var[i])
                elif type(var[i]) is list:
                    var[i] = [x for x in var[i] if x != 20]
                    for j in var[i]:
                        if type(j) is dict:
                            self.remove_nones(j)
