from urllib.parse import urlencode

from Apifon.sdk.Resource.Helpers.AbstractFieldSerializable import AbstractFieldSerializable
from Apifon.sdk.Resource.Helpers.ListType import ListType
from Apifon.sdk.Resource.Helpers.PaginatedRequest import PaginatedRequest
from Apifon.sdk.Resource.Helpers.PaginatedRequestBuilder import PaginatedRequestBuilder


class ListQueryBuilder(PaginatedRequestBuilder, AbstractFieldSerializable):

    def __init__(self):
        super().__init__()
        self.type = None
        self.fields = set()

    def get_builder(self):
        return self

    def with_type(self, type_):
        if type_ is None or not ListType.has_value(type_):
            raise TypeError("Invalid type. Should be of type ListType.value")
        self.type = type_
        return self

    def to_query_param_request(self):
        return self.build().to_query_param_request()

    def build(self):
        list_query_request = ListQueryBuilder.ListQueryRequest(self.type, self.fields, self.size, self.page, self.sort,
                                                               self.sort_dir)
        return list_query_request

    def get_fields(self):
        return self.fields

    def get_path_id(self):
        raise NotImplementedError('Not supported yet.')

    def add_visible_fields(self, fields):
        if fields is None or not isinstance(fields, set):
            raise TypeError("Invalid fields. Should be of type Set ")
        self.fields.update(fields)
        return self

    def add_visible_field(self, field):
        if field is None or not isinstance(field, str):
            raise TypeError("Invalid field. Should be of type str")
        self.fields.add(field)
        return self

    class ListQueryRequest(PaginatedRequest):

        def __init__(self, type_, fields, size, page, sort, sort_dir):
            super().__init__(page, size, sort, sort_dir)
            self.type = type_
            self.fields = fields

        def to_query_param_request(self):
            queryParams = {}
            self.add_query_params(queryParams)
            if self.type is not None and ListType.has_value(self.type):
                queryParams["type"] = self.type
            if self.fields is not None and bool(self.fields):
                queryParams['fields'] = ','.join(self.fields)
            return '?'+urlencode(queryParams)


