import json

from Apifon.sdk.Resource.Helpers.AbstractHttpEtag import AbstractHttpEtag
from Apifon.sdk.Resource.Helpers.AbstractJsonSerializable import AbstractJsonSerializable
from Apifon.sdk.Resource.Helpers.ListType import ListType
from Apifon.sdk.Model.Request.CampaignDefaultsBuilder import CampaignDefaultsBuilder
from Apifon.sdk.Model.Request.GdprPropertiesBuilder import GdprPropertiesBuilder
from Apifon.sdk.Response.ListResponse import ListResponse


class ListRequestBuilder(AbstractHttpEtag):

    def __init__(self, type_, title, prefix, list_id):
        self.id = list_id
        self.type = type_
        self.title = title
        self.prefix = prefix
        self.gdpr_properties = None
        self.campaign_defaults = None
        self.description = None
        self.etag = None

    @staticmethod
    def get_update_list_request_builder(list_resp_or_id):
        if list_resp_or_id is None or not bool(list_resp_or_id):
            raise ValueError("list_resp_or_id is null")
        if isinstance(list_resp_or_id, ListResponse):
            if list_resp_or_id.id is None or not bool(list_resp_or_id.id):
                raise ValueError("id of provided list is null")
            list_to_update = ListRequestBuilder(None, list_resp_or_id.title,
                                                list_resp_or_id.prefix, list_resp_or_id.id)
            list_to_update.campaign_defaults = list_resp_or_id.campaign_defaults
            list_to_update.gdpr_properties = list_resp_or_id.gdpr_properties
            list_to_update.description = list_resp_or_id.description
            list_to_update.etag = list_resp_or_id.get_etag()
            return list_to_update

        elif isinstance(list_resp_or_id, str):
            list_to_update = ListRequestBuilder(None, None, None, list_resp_or_id)
            return list_to_update
        else:
            raise TypeError("Invalid list_resp_or_id. Should be of type ListResponse or str")

    # type if of type ListType.value
    @staticmethod
    def get_add_request_builder(type_, title, prefix):
        if type_ is None or not ListType.has_value(type_):
            raise TypeError("Invalid list type. Should be of type ListType.value")
        if title is None or not isinstance(title, str):
            raise TypeError("Invalid title. Should be of type str")
        if prefix is None or not isinstance(prefix, str):
            raise TypeError("Invalid prefix. Should be of type str")
        return ListRequestBuilder(type_, title, prefix, None)

    def with_title(self, title):
        if title is None or not isinstance(title, str):
            raise TypeError("Invalid title. Should be of type str")
        self.title = title
        return self

    def with_prefix(self, prefix):
        if prefix is None or not isinstance(prefix, str):
            raise TypeError("Invalid prefix. Should be of type str")
        self.prefix = prefix
        return self

    def with_gdpr_properties(self, gdpr_properties):
        if isinstance(gdpr_properties, GdprPropertiesBuilder):
            self.gdpr_properties = gdpr_properties.build()
        elif isinstance(gdpr_properties, GdprPropertiesBuilder.GdprProperties):
            self.gdpr_properties = gdpr_properties
        else:
            raise TypeError("Invalid gdpr properties. Should be of type GdprPropertiesBuilder.GdprProperties or "
                            "GdprPropertiesBuilder ")
        return self

    def with_campaign_defaults(self, campaign_defaults):
        if isinstance(campaign_defaults, CampaignDefaultsBuilder):
            self.campaign_defaults = campaign_defaults.build()
        elif isinstance(campaign_defaults, CampaignDefaultsBuilder.CampaignDefaults):
            self.campaign_defaults = campaign_defaults
        else:
            raise TypeError("Invalid campaign defaults. Should be of type CampaignDefaultsBuilder.CampaignDefaults or "
                            "CampaignDefaultsBuilder")
        return self

    def with_description(self, description):
        if not isinstance(description, str):
            raise TypeError("Invalid description. Should be of type str")
        self.description = description
        return self

    def with_etag(self, etag):
        if not isinstance(etag, str):
            raise TypeError("Invalid etag. Should be of type str")
        self.etag = etag
        return self

    def build(self):

        if isinstance(self.type, ListType):
            self.type = self.type.value

        return self.ListRequest(self.id, self.type, self.title, self.prefix, self.gdpr_properties,
                                self.campaign_defaults, self.description, self.etag)

    # override
    def get_etag(self):
        return self.etag

    class ListRequest(AbstractHttpEtag, AbstractJsonSerializable):

        def __init__(self, id_, type_, title, prefix, gdpr_properties, campaign_defaults, description, etag):
            self.id = id_
            self.type = type_
            self.title = title
            self.prefix = prefix
            self.gdpr_properties = gdpr_properties
            self.campaign_defaults = campaign_defaults
            self.description = description
            self.etag = etag

        # override
        def get_etag(self):
            return self.etag

        # override
        def get_body(self):

            json_string = json.dumps(self, default=lambda o: o.__dict__)
            json_dict = json.loads(json_string)
            self.remove_nones(json_dict)

            # because 'from' is a reserved keyword, this changes 'from_' key to the the proper 'from'
            if 'campaign_defaults' in json_dict:
                if isinstance(json_dict['campaign_defaults'], dict) and 'from_' in json_dict['campaign_defaults']:
                    json_dict['campaign_defaults']['from'] = json_dict['campaign_defaults'].pop('from_')

            json_string = json.dumps(json_dict)
            return json_string

        def remove_nones(self, var):
            for i in list(var):
                if var[i] is None or (isinstance(var[i], dict) and not bool(var[i])):
                    var.pop(i)
                elif type(var[i]) is dict:
                    self.remove_nones(var[i])
                elif type(var[i]) is list:
                    var[i] = [x for x in var[i] if x != 20]
                    for j in var[i]:
                        if type(j) is dict:
                            self.remove_nones(j)
