import json

from Apifon.sdk.Resource.Helpers.AbstractHttpEtag import AbstractHttpEtag
from Apifon.sdk.Resource.Helpers.AbstractJsonSerializable import AbstractJsonSerializable
from Apifon.sdk.Resource.Helpers.SubscriberStatus import SubscriberStatus
from Apifon.sdk.Model.SubscriberGdprField import SubscriberGdprField
from Apifon.sdk.Response.SubscriberResponse import SubscriberResponse


class SubscriberRequestBuilder(AbstractHttpEtag):

    def __init__(self, list_id):
        self.list_id = list_id
        self.subscriber_fields = {}
        self.gdpr_fields = {}
        self.groupOptions = []
        self.etag = None
        self.destination = None
        self.status = None
        self.email = None
        self.email_status = None
        self.lookupMD5 = None

    @staticmethod
    def get_builder_from_subscriber(subscriber_response):
        if subscriber_response is None:
            raise ValueError("subscriber response is null")
        if not isinstance(subscriber_response, SubscriberResponse):
            raise TypeError("Invalid subscriber response. Should be of type SubscriberResponse")
        subscriber_builder = SubscriberRequestBuilder(subscriber_response.list_id)
        subscriber_builder.destination = subscriber_response.destination
        subscriber_builder.status = subscriber_response.status
        subscriber_builder.email = subscriber_response.email
        subscriber_builder.email_status = subscriber_response.email_status
        subscriber_builder.subscriber_fields = subscriber_response.subscriber_fields
        subscriber_builder.gdpr_fields = subscriber_response.gdpr_fields
        if subscriber_response.destination_id is None:
            subscriber_builder.lookupMD5 = subscriber_response.email_id
        else:
            subscriber_builder.lookupMD5 = subscriber_response.destination_id
        return subscriber_builder

    @staticmethod
    def get_add_subscriber_request_builder(list_id):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        return SubscriberRequestBuilder(list_id)

    def with_lookupMD5(self, lookupMD5):
        if lookupMD5 is None or not isinstance(lookupMD5, str):
            raise TypeError("Invalid lookupMD5. Should be of type str")
        self.lookupMD5 = lookupMD5
        return self

    # status should be of type SubscriberStatus.value
    def with_destination(self, destination, status):
        if destination is None or not isinstance(destination, str):
            raise TypeError("Invalid destination. Should be of type str")
        if status is None or not SubscriberStatus.has_value(status):
            raise TypeError("Invalid status. Should be of type SubscriberStatus.value")
        self.destination = destination
        self.status = status
        return self

    # email status should be of type SubscriberStatus.value
    def with_email(self, email, email_status):
        if email is None or not isinstance(email, str):
            raise TypeError("Invalid email. Should be of type str")
        if email_status is None or not SubscriberStatus.has_value(email_status):
            raise TypeError("Invalid email_status. Should be of type SubscriberStatus.value")
        self.email_status = email_status
        self.email = email
        return self

    # status should be of type SubscriberStatus.value
    def with_status(self, status):
        if status is None or not SubscriberStatus.has_value(status):
            raise TypeError("Invalid status. Should be of type SubscriberStatus.value")
        self.status = status
        return self

    #email status should be of type SubscriberStatus.value
    def with_email_status(self, email_status):
        if email_status is None or not SubscriberStatus.has_value(email_status):
            raise TypeError("Invalid email_status. Should be of type SubscriberStatus.value")
        self.email_status = email_status
        return self

    def with_etag(self, etag):
        if etag is None or not isinstance(etag, str):
            raise TypeError("Invalid etag. Should be of type str")
        self.etag = etag
        return self

    def clear_subscriber_fields(self):
        self.subscriber_fields.clear()
        return self

    def with_subscriber_field(self, field, value):
        if field is None or not isinstance(field, str):
            raise TypeError("Invalid field. Should be of type str")
        if value is None:
            raise ValueError("value is None")
        self.subscriber_fields[field] = value
        return self

    def with_subscriber_fields(self, subscriber_fields):
        if subscriber_fields is None or not isinstance(subscriber_fields, dict):
            raise TypeError("Invalid subscriber_fields. Should be of type dict")
        self.subscriber_fields.update(subscriber_fields)
        return self

    def clear_gdpr_fields(self):
        self.gdpr_fields.clear()
        return self

    def with_gdpr_field(self, field, gdpr_compliance):
        if field is None or not isinstance(field, str):
            raise TypeError("Invalid field. Should be of type str")
        if gdpr_compliance is None or not isinstance(gdpr_compliance, bool):
            raise TypeError("Invalid gdpr_compliance. Should be of type bool")
        if field in self.gdpr_fields:
            self.gdpr_fields[field].mobile = gdpr_compliance
            self.gdpr_fields[field].email = gdpr_compliance
        else:
            new_gdpr_field = SubscriberGdprField()
            new_gdpr_field.email = gdpr_compliance
            new_gdpr_field.mobile = gdpr_compliance
            self.gdpr_fields[field] = new_gdpr_field
        return self

    def with_mobile_gdpr_field(self, field, gdpr_compliance):
        if field is None or not isinstance(field, str):
            raise TypeError("Invalid field. Should be of type str")
        if gdpr_compliance is None or not isinstance(gdpr_compliance, bool):
            raise TypeError("Invalid gdpr_compliance. Should be of type bool")
        if field in self.gdpr_fields:
            self.gdpr_fields[field].mobile = gdpr_compliance
        else:
            new_gdpr_field = SubscriberGdprField()
            new_gdpr_field.mobile = gdpr_compliance
            self.gdpr_fields[field] = new_gdpr_field
        return self

    def with_email_gdpr_field(self, field, gdpr_compliance):
        if field is None or not isinstance(field, str):
            raise TypeError("Invalid field. Should be of type str")
        if gdpr_compliance is None or not isinstance(gdpr_compliance, bool):
            raise TypeError("Invalid gdpr_compliance. Should be of type bool")
        if field in self.gdpr_fields:
            self.gdpr_fields[field].email = gdpr_compliance
        else:
            new_gdpr_field = SubscriberGdprField()
            new_gdpr_field.email = gdpr_compliance
            self.gdpr_fields[field] = new_gdpr_field
        return self

    def with_gdpr_fields(self, gdpr_fields):
        if gdpr_fields is None or not isinstance(gdpr_fields, dict):
            raise TypeError("Invalid gdpr_fields. Should be of type dict")
        for subscriber_gdpr_field in gdpr_fields.values():
            if subscriber_gdpr_field is None or not isinstance(subscriber_gdpr_field, SubscriberGdprField):
                raise TypeError("Invalid gdpr_fields values. Should be of type SubscriberGdprField")
        self.gdpr_fields.update(gdpr_fields)
        return self

    def with_groupOptions(self,groupOptions):
        if groupOptions is None or not isinstance(groupOptions,list):
            raise TypeError("Invalid groupOptions. Should be of type list")
        self.groupOptions = groupOptions
        return self

    def get_etag(self):
        return self.etag

    def build(self):
        return SubscriberRequestBuilder.SubscriberRequest(self.list_id, self.subscriber_fields, self.gdpr_fields,
                                                          self.etag, self.destination, self.status, self.email,
                                                          self.email_status, self.lookupMD5,self.groupOptions)

    class SubscriberRequest(AbstractJsonSerializable, AbstractHttpEtag):

        def __init__(self, list_id, subscriber_fields, gdpr_fields, etag, destination, status, email, email_status,
                     lookupMD5,groupOptions):
            self.list_id = list_id
            self.subscriber_fields = subscriber_fields
            self.gdpr_fields = gdpr_fields
            self.groupOptions = groupOptions
            self.etag = etag
            self.destination = destination
            self.status = status
            self.email = email
            self.email_status = email_status
            self.lookupMD5 = lookupMD5

        # override
        def get_etag(self):
            return self.etag

        # override
        def get_body(self):

            for k, v in self.subscriber_fields.items():
                self.subscriber_fields[k] = str(v)

            json_string = json.dumps(self, default=lambda o: o.__dict__)
            json_dict = json.loads(json_string)
            self.remove_nones(json_dict)
            json_string = json.dumps(json_dict)
            return json_string

        def remove_nones(self, var):
            for i in list(var):
                if var[i] is None or (isinstance(var[i], dict) and not bool(var[i])):
                    var.pop(i)
                elif type(var[i]) is dict:
                    self.remove_nones(var[i])
                elif type(var[i]) is list:
                    var[i] = [x for x in var[i] if x != 20]
                    for j in var[i]:
                        if type(j) is dict:
                            self.remove_nones(j)
