from abc import ABCMeta, abstractmethod
import Apifon.sdk.Mookee as MookeeFolder

from Apifon.sdk.Exception.ApifonRestError import ApifonRestError



class AbstractResource(metaclass=ABCMeta):

    @abstractmethod
    def __init__(self, endpoint):
        self.endpoint = endpoint
        self.cur_endpoint = None
        self.cur_body = None
        self.cur_method = None
        self.cur_headers = {}

    def add_etag_header(self, etag):
        if etag is None:
            return
        self.cur_headers['If-Match'] = etag

    def dispatch(self):
        return MookeeFolder.Mookee.dispatch(self)

    def dispatch_to_response(self):
        try:
            return MookeeFolder.Mookee.dispatch_to_response(self)
        except ApifonRestError:
            raise

    def add_headers(self, headers):
        if self.cur_headers is not None and not bool(self.cur_headers):
            headers.update(self.cur_headers)
            return headers

    def create(self, request):
        self.cur_method = "POST"
        self.cur_endpoint = self.endpoint + "/create"
        self.cur_body = request.get_create_body()

    def read(self, request):
        self.cur_method = "GET"
        self.cur_endpoint = self.endpoint + "/read"
        self.cur_body = request.get_create_body()

    def update(self, request):
        self.cur_method = "POST"
        self.cur_endpoint = self.endpoint + "/update"
        self.cur_body = request.get_create_body()

    def delete(self, request):
        self.cur_method = "DELETE"
        self.cur_endpoint = self.endpoint + "/delete"
        self.cur_body = request.get_create_body()

    def is_successful(self, response):
        if response.status == 304:
            print("Status: 304\n"
                  "Title: Not modified. The request was completed but the resource was not modified")
        return True

