import json
import os
from urllib.parse import urlsplit, SplitResult

from Apifon.sdk.Model.Request.GroupRequestBuilder import GroupRequestBuilder
from Apifon.sdk.Model.Request.GroupSingleQuery import GroupSingleQuery
from Apifon.sdk.Resource.AbstractResource import AbstractResource
from Apifon.sdk.Resource.UriLocator import UriLocator
from Apifon.sdk.Response.GroupResponse import GroupResponse


class GroupResource(AbstractResource, UriLocator):

    def __init__(self, endpoint="/services/api/v1/list/{list_id}/group"):
        super().__init__(endpoint)

    def replace_placeholder_and_set_url(self, endpoint, list_id):
        self.cur_endpoint = endpoint.replace("{list_id}", list_id)

    def add_group(self, group_request):
        if not isinstance(group_request, GroupRequestBuilder.GroupRequest):
            raise TypeError("Invalid request. Should be of type GroupRequestBuilder.GroupRequest")
        if group_request.list_id is None or not bool(group_request.list_id):
            raise ValueError("list id is null")
        self.cur_method = "POST"
        self.replace_placeholder_and_set_url(self.endpoint, group_request.list_id)
        self.cur_body = group_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url

    def get_group(self, group_single_query):
        if not isinstance(group_single_query, GroupSingleQuery):
            raise TypeError("Invalid request. Should be of type GroupSingleQuery")
        if group_single_query.list_id is None or not bool(group_single_query.list_id):
            raise ValueError("list id is null")
        if group_single_query.group_field is None or not bool(group_single_query.group_field):
            raise ValueError("group field is null")

        self.cur_method = "GET"
        self.replace_placeholder_and_set_url(self.endpoint + "/" + group_single_query.group_field,
                                             group_single_query.list_id)
        self.cur_body = ""
        response = self.dispatch_to_response()
        theGroup = json.loads(response.read().decode('utf8'))
        return GroupResponse(theGroup)

    def update_group(self, group_request):
        if not isinstance(group_request, GroupRequestBuilder.GroupRequest):
            raise TypeError("Invalid request. Should be of type GroupRequestBuilder.GroupRequest")
        if group_request.list_id is None or not bool(group_request.list_id):
            raise ValueError("list id is null")
        if group_request.group_field is None or not bool(group_request.group_field):
            raise ValueError("group field is null")
        self.cur_method = "PUT"
        self.replace_placeholder_and_set_url(self.endpoint + "/" + group_request.group_field,
                                             group_request.list_id)
        self.cur_body = group_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url

    def remove_group(self, list_id, group_field):
        if list_id is None or not bool(list_id):
            raise ValueError("list id is null")
        if group_field is None or not bool(group_field):
            raise ValueError("group field is null")
        self.cur_method = "DELETE"
        self.replace_placeholder_and_set_url(self.endpoint + "/" + group_field, list_id)
        self.cur_body = ""
        response = self.dispatch_to_response()
        return response.status == 204

    def get_by_resource_uri(self, resource_uri):
        if not isinstance(resource_uri, SplitResult):
             raise TypeError("Invalid uri. Should be of type urllib.parse SplitResult")

        self.cur_method = "GET"
        self.cur_endpoint = resource_uri.path
        self.cur_body = ""
        response = self.dispatch_to_response()
        theGroup = json.loads(response.read().decode('utf8'))
        return GroupResponse(theGroup)



    def get_list_by_resource_uri(self, resource_uri):
        raise NotImplementedError
