import base64
import hashlib
import secrets


class S256PKCEChallenge:
    def __init__(self, code_verifier):
        self.code_verifier = None
        self.code_challenge = None
        self.init_S256PKCE_challenge(code_verifier)

    def init_S256PKCE_challenge(self, code_verifier):
        try:
            result = base64.b64encode(hashlib.sha256(code_verifier.encode('US_ASCII')).digest())
            self.code_challenge = result.decode("US_ASCII").split("=")[0]
            self.code_verifier = code_verifier
        except Exception as c:
            raise ValueError(c)

    @staticmethod
    def get_random_S256PKCE_challenge():
        token1 = secrets.token_bytes()
        result = base64.b64encode(token1)
        return S256PKCEChallenge(result.decode("US_ASCII").split("=")[0])
