import json
import os
from urllib.parse import urlsplit, SplitResult

from Apifon.sdk.Exception.ApifonRestError import ApifonRestError
from Apifon.sdk.Resource.Helpers.PaginatedRequest import PaginatedRequest
from Apifon.sdk.Model.CollectionEntityResponse import CollectionEntityResponse
from Apifon.sdk.Model.Request.ListRequestBuilder import ListRequestBuilder
from Apifon.sdk.Model.Request.ListSingleQuery import ListSingleQuery
from Apifon.sdk.Resource.AbstractResource import AbstractResource
from Apifon.sdk.Resource.UriLocator import UriLocator
from Apifon.sdk.Response.ListResponse import ListResponse


class ListResource(AbstractResource, UriLocator):

    def __init__(self, endpoint="/services/api/v1/list"):
        super().__init__(endpoint)

    def create_list(self, list_request):
        if not isinstance(list_request, ListRequestBuilder.ListRequest):
            raise TypeError("Invalid request. Should be of type ListRequestBuilder.ListRequest")
        self.cur_method = "POST"
        self.cur_endpoint = self.endpoint + "/"
        self.cur_body = list_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url


    def update_list(self, list_request):
        if not isinstance(list_request, ListRequestBuilder.ListRequest):
            raise TypeError("Invalid request. Should be of type ListRequestBuilder.ListRequest")
        if list_request.id is None or not bool(list_request.id):
            raise ValueError("list id is null")

        self.cur_method = "PUT"
        self.cur_endpoint = self.endpoint + "/" + list_request.id
        self.cur_body = list_request.get_body()
        # self.add_etag_header(list_request.get_etag())
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("CONTENT-LOCATION"))
        return url


    def get_list(self, list_single_query):
        if not isinstance(list_single_query, ListSingleQuery):
            raise TypeError("Invalid request. Should be of type ListSingleQuery")
        if list_single_query.list_id is None or not bool(list_single_query.list_id):
            raise ValueError("list id is null")

        self.cur_method = "GET"
        self.cur_endpoint = self.endpoint + "/" + list_single_query.get_path_id() + list_single_query.get_query_params()
        self.cur_body = ""
        # self.add_etag_header(list_single_query.get_etag())
        response = self.dispatch_to_response()
        thelist = json.loads(response.read().decode('utf8'))
        return ListResponse(thelist)


    def get_lists(self, paginated_request):
        if paginated_request is None or not isinstance(paginated_request, PaginatedRequest) and not isinstance(paginated_request, SplitResult):
            raise TypeError("Invalid paginated request. Should be of type PaginatedRequest or urllib.parse SplitResult")

        if isinstance(paginated_request, PaginatedRequest):

            self.cur_method = "GET"
            self.cur_endpoint = self.endpoint + '/' + paginated_request.to_query_param_request()
            self.cur_body = ""
            response = self.dispatch_to_response()
            lists = json.loads(response.read().decode('utf8'))
            return CollectionEntityResponse(lists, 'list')

        if isinstance(paginated_request, SplitResult):
            return self.get_list_by_resource_uri(paginated_request)


    def remove_list(self, list_id):
        if list_id is None or not bool(list_id):
            raise ValueError("list id is null")

        self.cur_method = "DELETE"
        self.cur_endpoint = self.endpoint + '/' + list_id
        self.cur_body = ""
        response = self.dispatch_to_response()
        return response.status == 204


    def get_list_by_resource_uri(self, resource_uri):
        if not isinstance(resource_uri, SplitResult):
            raise TypeError("Invalid uri. Should be of type urllib.parse SplitResult")

        self.cur_method = "GET"
        query_params = ""
        if resource_uri.query is not None and bool(resource_uri.query):
            query_params = "?" + resource_uri.query
        self.cur_endpoint = resource_uri.path + query_params
        self.cur_body = ""
        response = self.dispatch_to_response()
        lists = json.loads(response.read().decode('utf8'))
        return CollectionEntityResponse(lists, 'list')


    def get_by_resource_uri(self, resource_uri):
        if not isinstance(resource_uri, SplitResult):
            raise TypeError("Invalid uri. Should be of type urllib.parse SplitResult")
        listId = os.path.basename(os.path.normpath(resource_uri.path))
        return self.get_list(ListSingleQuery.get_list_request(listId))

