import urllib.request
import urllib.parse
import urllib.error
import json

import Apifon.sdk.Mookee as mookee
from Apifon.sdk.Exception.OAuthTokenError import OAuthTokenError
from Apifon.sdk.Resource.AbstractResource import AbstractResource
from Apifon.sdk.Resource.Helpers.OAuthCredentials import OAuthCredentials
from Apifon.sdk.Resource.Helpers.OAuthGrantType import OAuthGrantType
from Apifon.sdk.Response.OauthTokenErrorResponse import OauthTokenErrorResponse


class OAuthResource(AbstractResource):

    def __init__(self, authorizationConfig):
        super().__init__(mookee.Mookee.get_instance().get_identity_server_url())
        self.authorize_url = "/oauth2/authorize"
        self.token_url = "/oauth2/token"
        self.revoke_url = "/oauth2/revoke"
        self.authorisation_config = authorizationConfig
        self.oauth_credentials = OAuthCredentials()

    def get_tokens_for_client_credentials_grant(self):
        if self.get_grand_type().value != OAuthGrantType.CLIENT_CREDENTIALS.value:
            raise OAuthTokenError("Tried to retrieve tokens for client credentials but grant type is not "
                                  "client_credentials")
        try:
            post_url = self.endpoint
            endpoint = post_url + self.token_url + "?grant_type=" + self.get_grand_type().get_grant_type_as_string() + \
                       "&client_id=" + self.authorisation_config.client_id + \
                       "&client_secret=" + self.authorisation_config.client_secret+\
                       "&scope=" + self.authorisation_config.get_scopes_as_string()
            headers = {'Content-type': 'application/x-www-form-urlencoded'}
            body = ""
            data = urllib.parse.urlencode(body).encode("ascii")
            req = urllib.request.Request(endpoint, data, headers)
            post_response = urllib.request.urlopen(req)
            if post_response.getcode() == 200:
                credentials = OAuthCredentials(json.loads(post_response.read().decode('utf8')))
                self.oauth_credentials.access_code = None
                self.oauth_credentials.access_token = credentials.access_token
                self.oauth_credentials.refresh_token = None
                self.oauth_credentials.set_expires_in(credentials.expires_in)
        except urllib.error.HTTPError as e:
            tokenError = OauthTokenErrorResponse(json.loads(e.read().decode("utf-8")))
            raise OAuthTokenError(e, tokenError)
        except IOError as x:
            raise OAuthTokenError(x)

    def get_grand_type(self):
        return self.authorisation_config.grant_type

    def revoke_tokens(self):
          self.revoke_access_token()


    def revoke_access_token(self):
        if self.oauth_credentials.access_token is not None:
            self.__revoke_token(self.oauth_credentials.access_token, "access_token")
            self.oauth_credentials.access_token = None
            return True
        return False

    def refresh_token(self):
        if self.get_grand_type() is None:
            return
        if self.get_grand_type().value == OAuthGrantType.CLIENT_CREDENTIALS.value:
            self.get_tokens_for_client_credentials_grant()

    def __revoke_token(self, token, token_type_hint):
        try:
            post_url = self.endpoint
            endpoint = post_url + self.revoke_url + \
                       "?client_id=" + self.authorisation_config.client_id + \
                       "&token=" + token + \
                       "&token_type_hint=" + token_type_hint
            if self.get_grand_type().value == OAuthGrantType.CLIENT_CREDENTIALS.value:
                endpoint += "&client_secret=" + self.authorisation_config.client_secret

            headers = {'Content-type': 'application/x-www-form-urlencoded'}
            body = ""
            data = urllib.parse.urlencode(body).encode("ascii")
            req = urllib.request.Request(endpoint, data, headers)
            urllib.request.urlopen(req)
        except urllib.error.HTTPError as e:
            tokenError = OauthTokenErrorResponse(json.loads(e.read().decode("utf-8")))
            raise OAuthTokenError("Failed to revoke token with exception: " + str(e), tokenError)
        except IOError as x:
            raise OAuthTokenError(x)
