import json
from urllib.parse import urlsplit, SplitResult

from Apifon.sdk.Model.Request.SubgroupRequestBuilder import SubgroupRequestBuilder
from Apifon.sdk.Model.Request.SubgroupSingleQuery import SubgroupSingleQuery
from Apifon.sdk.Resource.AbstractResource import AbstractResource
from Apifon.sdk.Resource.UriLocator import UriLocator
from Apifon.sdk.Response.SubgroupResponse import SubgroupResponse


class SubgroupResource(AbstractResource, UriLocator):

    def __init__(self, endpoint="/services/api/v1/list/{list_id}/{group_field}"):
        super().__init__(endpoint)

    def replace_placeholder_and_set_url(self, endpoint, list_id, group_field):
        endpoint = endpoint.replace("{list_id}", list_id)
        self.cur_endpoint = endpoint.replace("{group_field}", group_field)

    def add_subgroup(self, subgroup_request):
        if not isinstance(subgroup_request, SubgroupRequestBuilder.SubgroupRequest):
            raise TypeError("Invalid request. Should be of type SubgroupRequestBuilder.SubgroupRequest")
        if subgroup_request.list_id is None or not bool(subgroup_request.list_id):
            raise ValueError("list id is null")
        if subgroup_request.group_field is None or not bool(subgroup_request.group_field):
            raise ValueError("group field is null")
        self.cur_method = "POST"
        self.replace_placeholder_and_set_url(self.endpoint + "/option", subgroup_request.list_id,
                                             subgroup_request.group_field)
        self.cur_body = subgroup_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url

    def add_subgroups(self, subgroup_request):
        if not isinstance(subgroup_request, SubgroupRequestBuilder.SubgroupRequest):
            raise TypeError("Invalid request. Should be of type SubgroupRequestBuilder.SubgroupRequest")
        if subgroup_request.list_id is None or not bool(subgroup_request.list_id):
            raise ValueError("list id is null")
        if subgroup_request.group_field is None or not bool(subgroup_request.group_field):
            raise ValueError("group field is null")
        self.cur_method = "POST"
        self.replace_placeholder_and_set_url(self.endpoint + "/options", subgroup_request.list_id,
                                             subgroup_request.group_field)
        self.cur_body = subgroup_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url

    def get_subgroup(self, subgroup_single_query):
        if not isinstance(subgroup_single_query, SubgroupSingleQuery):
            raise TypeError("Invalid request. Should be of type SubgroupSingleQuery")
        if subgroup_single_query.list_id is None or not bool(subgroup_single_query.list_id):
            raise ValueError("list id is null")
        if subgroup_single_query.group_field is None or not bool(subgroup_single_query.group_field):
            raise ValueError("group field is null")
        if subgroup_single_query.options_field is None or not bool(subgroup_single_query.options_field):
            raise ValueError("options field is null")

        self.cur_method = "GET"
        self.replace_placeholder_and_set_url(self.endpoint + "/option/" + subgroup_single_query.options_field,
                                             subgroup_single_query.list_id, subgroup_single_query.group_field)
        self.cur_body = ""
        response = self.dispatch_to_response()
        theGroup = json.loads(response.read().decode('utf8'))
        return SubgroupResponse(theGroup)

    def update_subgroup(self, subgroup_request):
        if not isinstance(subgroup_request, SubgroupRequestBuilder.SubgroupRequest):
            raise TypeError("Invalid request. Should be of type SubgroupRequestBuilder.SubgroupRequest")
        if subgroup_request.list_id is None or not bool(subgroup_request.list_id):
            raise ValueError("list id is null")
        if subgroup_request.group_field is None or not bool(subgroup_request.group_field):
            raise ValueError("group field is null")
        if subgroup_request.options_field is None or not bool(subgroup_request.options_field):
            raise ValueError("options field is null")
        self.cur_method = "PUT"
        self.replace_placeholder_and_set_url(self.endpoint + "/" + subgroup_request.options_field, subgroup_request.list_id,
                                             subgroup_request.group_field)
        self.cur_body = subgroup_request.get_body()
        response = self.dispatch_to_response()
        url = urlsplit(response.getheader("LOCATION"))
        return url

    def remove_subgroup(self, list_id, group_field, options_field):
        if list_id is None or not bool(list_id):
            raise ValueError("list id is null")
        if group_field is None or not bool(group_field):
            raise ValueError("group field is null")
        if options_field is None or not bool(options_field):
            raise ValueError("options field is null")
        self.cur_method = "DELETE"
        self.replace_placeholder_and_set_url(self.endpoint + "/" + options_field, list_id, group_field)
        self.cur_body = ""
        response = self.dispatch_to_response()
        return response.status == 204

    def get_by_resource_uri(self, resource_uri):
        if not isinstance(resource_uri, SplitResult):
            raise TypeError("Invalid uri. Should be of type urllib.parse SplitResult")

        self.cur_method = "GET"
        self.cur_endpoint = resource_uri.path
        self.cur_body = ""
        response = self.dispatch_to_response()
        theSubgroup = json.loads(response.read().decode('utf8'))
        return SubgroupResponse(theSubgroup)

    def get_list_by_resource_uri(self, resource_uri):
        raise NotImplementedError
