import json
from urllib.parse import urlsplit, SplitResult

from Apifon.sdk.Exception.ApifonRestError import ApifonRestError
from Apifon.sdk.Resource.Helpers.PaginatedRequest import PaginatedRequest
from Apifon.sdk.Model.CollectionEntityResponse import CollectionEntityResponse
from Apifon.sdk.Model.Request.SubscriberConditionQueryRequestBuilder import SubscriberConditionQueryRequestBuilder
from Apifon.sdk.Model.Request.SubscriberRequestBuilder import SubscriberRequestBuilder
from Apifon.sdk.Resource.AbstractResource import AbstractResource
from Apifon.sdk.Model.Request.SubscriberSingleQuery import SubscriberSingleQuery
from Apifon.sdk.Resource.UriLocator import UriLocator
from Apifon.sdk.Response.SubscriberResponse import SubscriberResponse


class SubscriberResource(AbstractResource, UriLocator):

    def __init__(self, endpoint="/services/api/v1/list/{list_id}/subscriber"):
        super().__init__(endpoint)

    def replace_placeholder_and_set_url(self, endpoint, list_id):
        self.cur_endpoint = endpoint.replace("{list_id}", list_id)

    def add_subscriber(self, subscriber_request):
        if subscriber_request is None or not isinstance(subscriber_request, SubscriberRequestBuilder.SubscriberRequest):
            raise TypeError("Invalid subscriber request. Should be of type SubscriberRequestBuilder.SubscriberRequest")
        if subscriber_request.list_id is None or not bool(subscriber_request.list_id):
            raise ValueError("list id is null")
        try:
            self.cur_method = "POST"
            self.replace_placeholder_and_set_url(self.endpoint + "/", subscriber_request.list_id)
            self.cur_body = subscriber_request.get_body()
            response = self.dispatch_to_response()
            url = urlsplit(response.getheader("LOCATION"))
            return url
        except ApifonRestError:
            raise

    def update_subscriber(self, subscriber_request):
        if subscriber_request is None or not isinstance(subscriber_request, SubscriberRequestBuilder.SubscriberRequest):
            raise TypeError("Invalid subscriber request. Should be of type SubscriberRequestBuilder.SubscriberRequest")
        if subscriber_request.list_id is None or not bool(subscriber_request.list_id):
            raise ValueError("list id is null")
        if subscriber_request.lookupMD5 is None or not bool(subscriber_request.lookupMD5):
            raise ValueError("lookupMD5 is null")
        try:
            self.cur_method = "PUT"
            self.replace_placeholder_and_set_url(self.endpoint + "/" + subscriber_request.lookupMD5,
                                                 subscriber_request.list_id)
            self.cur_body = subscriber_request.get_body()
            # self.add_etag_header(subscriber_request.get_etag())
            response = self.dispatch_to_response()
            url = urlsplit(response.getheader("CONTENT-LOCATION"))
            return url
        except ApifonRestError:
            raise

    def get_subscriber(self, subscriber_single_query):
        if subscriber_single_query is None or not isinstance(subscriber_single_query, SubscriberSingleQuery):
            raise TypeError("Invalid subscriber single query. Should be of type SubscriberSingleQuery")
        if subscriber_single_query.list_id is None or not isinstance(subscriber_single_query.list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if subscriber_single_query.get_path_id() is None or not isinstance(subscriber_single_query.get_path_id(), str):
            raise TypeError("Invalid lookupMD5. Should be of type str")
        try:
            self.cur_method = "GET"
            self.replace_placeholder_and_set_url(self.endpoint + "/" + subscriber_single_query.get_path_id() +
                                                 subscriber_single_query.get_query_params(),
                                                 subscriber_single_query.list_id)
            self.cur_body = ""
            response = self.dispatch_to_response()
            theSub = json.loads(response.read().decode('utf8'))
            return SubscriberResponse(theSub)
        except ApifonRestError:
            raise
        except IOError as x:
            raise IOError("Exception while trying to parse response", x)

    def querry_subscribers_with_conditions(self, subscriber_cond_query_requets):
        if subscriber_cond_query_requets is None or not isinstance(subscriber_cond_query_requets,
                                                                   SubscriberConditionQueryRequestBuilder.SubscriberConditionQueryRequest):
            raise TypeError("Invalid subscriber condition query request. Should be of type "
                            "SubscriberConditionQueryRequest")
        if subscriber_cond_query_requets.list_id is None:
            raise ValueError("list id is empty")
        try:
            self.cur_method = "POST"
            self.replace_placeholder_and_set_url(self.endpoint + "/query", subscriber_cond_query_requets.list_id)
            self.cur_body = subscriber_cond_query_requets.get_body()
            response = self.dispatch_to_response()
            subs = json.loads(response.read().decode('utf8'))
            return CollectionEntityResponse(subs, 'subscribers')
        except ApifonRestError:
            raise
        except IOError as x:
            raise IOError("Exception while trying to parse response", x)

    def querry_subscribers(self, paginated_request):
        if paginated_request is None or not isinstance(paginated_request, SplitResult) and not isinstance(
                paginated_request, PaginatedRequest):
            raise TypeError("Invalid paginated request. Should be of type PaginatedRequest or urllib.parse SplitResult")
        if isinstance(paginated_request, PaginatedRequest):
            if paginated_request.get_path_id() is None or not isinstance(paginated_request.get_path_id(), str):
                raise TypeError("Invalid list id. Should be of type str")
            try:
                self.cur_method = "GET"
                self.replace_placeholder_and_set_url(self.endpoint + paginated_request.to_query_param_request(),
                                                     paginated_request.get_path_id())
                self.cur_body = ""
                response = self.dispatch_to_response()
                subs = json.loads(response.read().decode('utf8'))
                return CollectionEntityResponse(subs, 'subscribers')
            except ApifonRestError:
                raise
            except IOError as x:
                raise IOError("Exception while trying to parse response", x)
        if isinstance(paginated_request, SplitResult):
            try:
                return self.get_list_by_resource_uri(paginated_request)
            except ApifonRestError:
                raise
            except IOError as x:
                raise IOError("Exception while trying to parse response", x)

    def subscribe_subscriber(self, list_id, lookupMD5):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if lookupMD5 is None or not isinstance(lookupMD5, str):
            raise TypeError("Invalid lookupMD5. Should be of type str")
        try:
            self.cur_method = "POST"
            self.replace_placeholder_and_set_url(self.endpoint + "/" + lookupMD5 + "/subscribe", list_id)
            self.cur_body = ""
            response = self.dispatch_to_response()
            return self.is_successful(response) #return true?
        except ApifonRestError:
            raise

    def unsubscribe_subscriber(self, list_id, lookupMD5):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if lookupMD5 is None or not isinstance(lookupMD5, str):
            raise TypeError("Invalid lookupMD5. Should be of type str")
        try:
            self.cur_method = "POST"
            self.replace_placeholder_and_set_url(self.endpoint + "/" + lookupMD5 + "/unsubscribe", list_id)
            self.cur_body = ""
            response = self.dispatch_to_response()
            return self.is_successful(response) #return true?
        except ApifonRestError:
            raise

    def remove_subscriber(self, list_id, lookupMD5):
        if list_id is None or not isinstance(list_id, str):
            raise TypeError("Invalid list id. Should be of type str")
        if lookupMD5 is None or not isinstance(lookupMD5, str):
            raise TypeError("Invalid lookupMD5. Should be of type str")
        try:
            self.cur_method = "DELETE"
            self.replace_placeholder_and_set_url(self.endpoint + "/" + lookupMD5, list_id)
            self.cur_body = ""
            response = self.dispatch_to_response()
            return response.status == 204
        except ApifonRestError:
            raise

    def get_by_resource_uri(self, resource_uri):
        if not isinstance(resource_uri, SplitResult):
            raise TypeError("Invalid uri. Should be of type urllib.parse SplitResult")
        try:
            self.cur_method = "GET"
            query_params = ""
            if resource_uri.query is not None and bool(resource_uri.query):
                query_params = "?" + resource_uri.query
            self.cur_endpoint = resource_uri.path + query_params
            self.cur_body = ""
            response = self.dispatch_to_response()
            sub = json.loads(response.read().decode('utf8'))
            return SubscriberResponse(sub)
        except ApifonRestError:
            raise
        except IOError as x:
            raise IOError("Exception while trying to parse response", x)

    def get_list_by_resource_uri(self, resource_uri):
        if not isinstance(resource_uri, SplitResult):
            raise TypeError("Invalid uri. Should be of type urllib.parse SplitResult")
        try:
            self.cur_method = "GET"
            query_params = ""
            if resource_uri.query is not None and bool(resource_uri.query):
                query_params = "?" + resource_uri.query
            self.cur_endpoint = resource_uri.path + query_params
            self.cur_body = ""
            response = self.dispatch_to_response()
            subs = json.loads(response.read().decode('utf8'))
            return CollectionEntityResponse(subs, 'subscribers')
        except ApifonRestError:
            raise
        except IOError as x:
            raise IOError("Exception while trying to parse response", x)
